@@ -9,11 +9,14 @@ import urllib
 import warnings
 from datetime import datetime as dt,timedelta
 import requests
+from ftplib import FTP
 
 from ..tracks import *
 from ..tracks.tools import *
+from ..tracks.plot import TrackPlot
 from ..utils import *
 from .. import constants
+from ..recon import ReconDataset
 
 try:
     import zipfile
@@ -46,6 +49,57 @@ class RealtimeStorm(Storm):
     -------
     RealtimeStorm
         Instance of a RealtimeStorm object.
+    
+    Notes
+    -----
+    A RealtimeStorm object is retrieved from a Realtime object's ``get_storm()`` method, or directly as an attribute of the Realtime object. For example, if an active storm has an ID of 'EP012022', it can be retrieved as such:
+    
+    .. code-block:: python
+    
+        from tropycal import realtime
+        realtime_obj = realtime.Realtime()
+        storm = realtime_obj.get_storm('EP012022')
+    
+    Now this storm's data is stored in the variable ``storm``, which is an instance of RealtimeStorm and can access all of the methods and attributes of a RealtimeStorm object.
+    
+    All the variables associated with a RealtimeStorm object (e.g., lat, lon, date, vmax) can be accessed in two ways. The first is directly from the RealtimeStorm object:
+    
+    >>> storm.lat
+    array([ 9.8, 10.3, 10.8, 11.4, 11.9, 12.1, 12.2, 12.4, 12.6, 12.8, 13. ,
+           12.9, 12.8, 12.9, 13.2, 13.6, 13.8, 13.9, 14. , 14. , 14.3, 14.6,
+           15.1, 15.4])
+    
+    The second is via ``storm.vars``, which returns a dictionary of the variables associated with the RealtimeStorm object. This is also a quick way to access all of the variables associated with a RealtimeStorm object:
+    
+    >>> variable_dict = storm.vars
+    >>> lat = variable_dict['lat']
+    >>> lon = variable_dict['lon']
+    >>> print(variable_dict.keys())
+    dict_keys(['date', 'extra_obs', 'special', 'type', 'lat', 'lon', 'vmax', 'mslp', 'wmo_basin'])
+    
+    RealtimeStorm objects also have numerous attributes with information about the storm. ``storm.attrs`` returns a dictionary of the attributes for this RealtimeStorm object.
+    
+    It should be noted that RealtimeStorm objects have additional attributes that Storm objects do not, specifically for 2 and 5 day NHC formation probability. These only display values for invests within NHC's area of responsibility; tropical cyclones or invests in JTWC's area of responsibility display "N/A".
+    
+    >>> print(storm.attrs)
+    {'id': 'EP012022',
+     'operational_id': 'EP012022',
+     'name': 'AGATHA',
+     'year': 2022,
+     'season': 2022,
+     'basin': 'east_pacific',
+     'source_info': 'NHC Hurricane Database',
+     'invest': False,
+     'source_method': "NHC's Automated Tropical Cyclone Forecasting System (ATCF)",
+     'source_url': 'https://ftp.nhc.noaa.gov/atcf/btk/',
+     'source': 'hurdat',
+     'ace': 6.055,
+     'prob_2day': 'N/A',
+     'prob_5day': 'N/A',
+     'risk_2day': 'N/A',
+     'risk_5day': 'N/A',
+     'realtime': True}
+    
     """
     
     def __setitem__(self, key, value):
@@ -102,10 +156,10 @@ class RealtimeStorm(Storm):
         
         #Add additional information
         summary.append("\nMore Information:")
-        add_space = np.max([len(key) for key in self.coords.keys()])+3
-        for key in self.coords.keys():
+        add_space = np.max([len(key) for key in self.attrs.keys()])+3
+        for key in self.attrs.keys():
             key_name = key+":"
-            val = '%0.1f'%(self.coords[key]) if key == 'ace' else self.coords[key]
+            val = '%0.1f'%(self.attrs[key]) if key == 'ace' else self.attrs[key]
             summary.append(f'{" "*4}{key_name:<{add_space}}{val}')
 
         return "\n".join(summary)
@@ -117,13 +171,13 @@ class RealtimeStorm(Storm):
         
         #Add other attributes about the storm
         keys = self.dict.keys()
-        self.coords = {}
+        self.attrs = {}
         self.vars = {}
         for key in keys:
             if key == 'realtime': continue
             if isinstance(self.dict[key], list) == False and isinstance(self.dict[key], dict) == False:
                 self[key] = self.dict[key]
-                self.coords[key] = self.dict[key]
+                self.attrs[key] = self.dict[key]
             if isinstance(self.dict[key], list) == True and isinstance(self.dict[key], dict) == False:
                 self.vars[key] = np.array(self.dict[key])
                 self[key] = np.array(self.dict[key])
@@ -132,19 +186,43 @@ class RealtimeStorm(Storm):
         if stormTors is not None and isinstance(stormTors,dict):
             self.stormTors = stormTors['data']
             self.tornado_dist_thresh = stormTors['dist_thresh']
-            self.coords['Tornado Count'] = len(stormTors['data'])
+            self.attrs['Tornado Count'] = len(stormTors['data'])
         
         #Get Archer track data for this storm, if it exists
-        self.get_archer()
-        
+        try:
+            self.get_archer()
+        except:
+            pass
+    
+        #Initialize recon dataset instance
+        self.recon = ReconDataset(storm=self)
+
         #Determine if storm object was retrieved via realtime object
         if 'realtime' in keys and self.dict['realtime']:
             self.realtime = True
-            self.coords['realtime'] = True
+            self.attrs['realtime'] = True
         else:
             self.realtime = False
-            self.coords['realtime'] = False
+            self.attrs['realtime'] = False
             
+    def get_realtime_formation_prob(self):
+        
+        r"""
+        Retrieve the latest NHC formation probability. Only valid for invests within NHC's area of responsibility.
+        
+        Returns
+        -------
+        dict
+            Dictionary containing latest NHC forecast formation probability, if available. If none, defaults to zero or N/A.
+        """
+        
+        return {
+            'prob_2day': self.dict['prob_2day'],
+            'risk_2day': self.dict['risk_2day'],
+            'prob_5day': self.dict['prob_5day'],
+            'risk_5day': self.dict['risk_5day']
+        }
+    
     def download_graphic_realtime(self,save_path=""):
         
         r"""
@@ -256,6 +334,11 @@ class RealtimeStorm(Storm):
         r"""
         Retrieve a dictionary containing the latest official forecast. Available for both NHC and JTWC sources.
         
+        Parameters
+        ----------
+        ssl_certificate : boolean, optional
+            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from the default JTWC source ("jtwc"). Default is True. Use False *ONLY* if True causes an SSL certification error.
+        
         Returns
         -------
         dict
@@ -274,16 +357,13 @@ class RealtimeStorm(Storm):
         if self.source == 'hurdat':
         
             #Get forecast for this storm
-            url = f"https://ftp.nhc.noaa.gov/atcf/fst/{self.id.lower()}.fst"
-            if requests.get(url).status_code != 200: raise RuntimeError("NHC forecast data is unavailable for this storm.")
-
-            #Read file content
-            f = urllib.request.urlopen(url)
-            content = f.read()
-            content = content.decode("utf-8")
-            content = content.split("\n")
-            content = [(i.replace(" ","")).split(",") for i in content]
-            f.close()
+            try:
+                content = read_url(f"https://ftp.nhc.noaa.gov/atcf/fst/{self.id.lower()}.fst")
+            except:
+                try:
+                    content = read_url(f"ftp://ftp.nhc.noaa.gov/atcf/fst/{self.id.lower()}.fst")
+                except:
+                    raise RuntimeError("NHC forecast data is unavailable for this storm.")
 
             #Iterate through every line in content:
             forecasts = {}
@@ -476,7 +556,7 @@ class RealtimeStorm(Storm):
         return self.latest_forecast
 
     def plot_forecast_realtime(self,track_labels='fhr',cone_days=5,domain="dynamic_forecast",
-                                   ax=None,cartopy_proj=None,save_path=None,prop={},map_prop={},ssl_certificate=True):
+                                   ax=None,cartopy_proj=None,save_path=None,ssl_certificate=True,**kwargs):
         
         r"""
         Plots the latest available official forecast. Available for both NHC and JTWC sources.
@@ -500,6 +580,11 @@ class RealtimeStorm(Storm):
             Instance of a cartopy projection to use. If none, one will be generated. Default is none.
         save_path : str
             Relative or full path of directory to save the image in. If none, image will not be saved.
+        
+        Other Parameters
+        ----------------
+        ssl_certificate : boolean, optional
+            If a JTWC forecast, this determines whether to disable SSL certificate when retrieving data from the default JTWC source ("jtwc"). Default is True. Use False *ONLY* if True causes an SSL certification error.
         prop : dict
             Property of storm track lines.
         map_prop : dict
@@ -511,6 +596,10 @@ class RealtimeStorm(Storm):
             Instance of axes containing the plot is returned.
         """
         
+        #Retrieve kwargs
+        prop = kwargs.pop('prop',{})
+        map_prop = kwargs.pop('map_prop',{})
+        
         #Check to ensure storm is not an invest
         if self.invest:
             raise RuntimeError("Error: NHC does not issue advisories for invests that have not been designated as Potential Tropical Cyclones.")
@@ -578,8 +667,9 @@ class RealtimeStorm(Storm):
         
         #Check to ensure storm is not an invest
         if self.invest:
-            msg = "NHC does not issue public advisories on invests. Defaulting to best track method."
-            warnings.warn(msg)
+            if self.source == 'hurdat':
+                msg = "NHC does not issue public advisories on invests. Defaulting to best track method."
+                warnings.warn(msg)
             source = 'best_track'
         
         #Declare empty dict
@@ -592,11 +682,10 @@ class RealtimeStorm(Storm):
                 latest_btk = self.date[-1]
                 
                 #Get latest available public advisory
-                f = urllib.request.urlopen(f"https://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml")
-                content = f.read()
-                content = content.decode("utf-8")
-                content = content.split("\n")
-                f.close()
+                try:
+                    content = read_url(f"https://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml",subsplit=False)
+                except:
+                    content = read_url(f"ftp://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml",subsplit=False)
                 
                 #Get UTC time of advisory
                 results = [i for i in content if 'messageDateTimeUTC' in i][0]
@@ -618,11 +707,10 @@ class RealtimeStorm(Storm):
             current_advisory['source'] = 'NHC Public Advisory'
 
             #Get latest available public advisory
-            f = urllib.request.urlopen(f"https://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml")
-            content = f.read()
-            content = content.decode("utf-8")
-            content = content.split("\n")
-            f.close()
+            try:
+                content = read_url(f"https://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml",subsplit=False)
+            except:
+                content = read_url(f"ftp://ftp.nhc.noaa.gov/atcf/adv/{self.id.lower()}_info.xml",subsplit=False)
 
             #Get public advisory number
             results = [i for i in content if 'advisoryNumber' in i][0]
@@ -817,15 +905,22 @@ class RealtimeStorm(Storm):
 
         #Get list of all public advisories for this storm
         url_disco = 'https://ftp.nhc.noaa.gov/atcf/pub/'
-        page = requests.get(url_disco).text
-        content = page.split("\n")
-        files = []
-        for line in content:
-            if ".public" in line and self.id.lower() in line:
-                filename = line.split('">')[1]
-                filename = filename.split("</a>")[0]
-                files.append(filename)
-        del content
+        try:
+            page = requests.get(url_disco).text
+            content = page.split("\n")
+            files = []
+            for line in content:
+                if ".public" in line and self.id.lower() in line:
+                    filename = line.split('">')[1]
+                    filename = filename.split("</a>")[0]
+                    files.append(filename)
+            del content
+        except:
+            ftp = FTP('ftp.nhc.noaa.gov')
+            ftp.login()
+            ftp.cwd('atcf/pub')
+            files = ftp.nlst()
+            out = ftp.quit()
 
         #Keep only largest number
         numbers = [int(i.split(".")[-1]) for i in files]
@@ -849,11 +944,7 @@ class RealtimeStorm(Storm):
             files = [i for i in files if f".public_{max_letter}" in i]
 
         #Read file containing advisory
-        f = urllib.request.urlopen(url_disco + files[0])
-        content = f.read()
-        content = content.decode("utf-8")
-        content = content.split("\n")
-        f.close()
+        content = read_url(url_disco + files[0],subsplit=False)
 
         #Figure out time issued
         hr = content[6].split(" ")[0]
