@@ -17,6 +17,10 @@ import matplotlib.colors as mcolors
 import matplotlib as mlib
 import warnings
 import scipy.interpolate as interp
+import re
+import shapefile
+import zipfile
+from io import StringIO, BytesIO
 
 from .. import constants
 
@@ -403,7 +407,7 @@ def accumulated_cyclone_energy(wind_speed,hours=6):
 def dropsonde_mslp_estimate(mslp,surface_wind):
     
     r"""
-    Apply a NHC rule of thumb for estimating a TC's minimum central mean sea level pressure (MSLP) from a dropsonde released in the eye, accounting for drifting by factoring in the surface wind in knots.
+    Apply a NHC rule of thumb for estimating a TC's minimum central MSLP. This is intended for a dropsonde released in the eye, accounting for drifting by factoring in the surface wind in knots.
     
     Parameters
     ----------
@@ -425,6 +429,140 @@ def dropsonde_mslp_estimate(mslp,surface_wind):
     
     return mslp - (surface_wind / 10.0)
 
+def get_two_current():
+    
+    r"""
+    Retrieve the latest NHC Tropical Weather Outlook (TWO).
+    
+    Returns
+    -------
+    dict
+        A dictionary of shapefiles for points, areas and lines.
+    
+    Notes
+    -----
+    The shapefiles returned are modified versions of Cartopy's BasicReader, allowing to read in shapefiles directly from URL without having to download the shapefile locally first.
+    """
+    
+    #Retrieve NHC shapefiles for development areas
+    shapefiles = {}
+    for name in ['areas','lines','points']:
+
+        try:
+            #Read in shapefile zip from NHC
+            url = 'https://www.nhc.noaa.gov/xgtwo/gtwo_shapefiles.zip'
+            request = urllib.request.Request(url)
+            response = urllib.request.urlopen(request)
+            file_like_object = BytesIO(response.read())
+            tar = zipfile.ZipFile(file_like_object)
+
+            #Get file list (points, areas)
+            members = '\n'.join([i for i in tar.namelist()])
+            nums = "[0123456789]"
+            search_pattern = f'gtwo_{name}_20{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}.shp'
+            pattern = re.compile(search_pattern)
+            filelist = pattern.findall(members)
+            files = []
+            for file in filelist:
+                if file not in files: files.append(file.split(".shp")[0]) #remove duplicates
+
+            #Retrieve necessary components for shapefile
+            members = tar.namelist()
+            members_names = [i for i in members]
+            data = {'shp':0,'dbf':0,'prj':0,'shx':0}
+            for key in data.keys():
+                idx = members_names.index(files[0]+"."+key)
+                data[key] = BytesIO(tar.read(members[idx]))
+
+            #Read in shapefile
+            orig_reader = shapefile.Reader(shp=data['shp'], dbf=data['dbf'], prj=data['prj'], shx=data['shx'])
+            shapefiles[name] = BasicReader(orig_reader)
+        except:
+            shapefiles[name] = None
+    
+    return shapefiles
+    
+def get_two_archive(time):
+    
+    r"""
+    Retrieve an archived NHC Tropical Weather Outlook (TWO). If none available within 30 hours of the specified date, an empty dict is returned.
+    
+    Parameters
+    ----------
+    time : datetime
+        Valid time for archived shapefile.
+    
+    Returns
+    -------
+    dict
+        A dictionary of shapefiles for points, areas and lines.
+    
+    Notes
+    -----
+    The shapefiles returned are modified versions of Cartopy's BasicReader, allowing to read in shapefiles directly from URL without having to download the shapefile locally first.
+    
+    TWO shapefiles are available courtesy of the National Hurricane Center beginning 28 July 2010.
+    """
+    
+    #Find closest NHC shapefile if within 24 hours
+    url = 'https://www.nhc.noaa.gov/gis/archive_gtwo.php'
+    page = requests.get(url).text
+    content = page.split("\n")
+    files = []
+    for line in content:
+        if '<a href="gtwo/archive/2' in line:
+            filename = line.split('zip">')[1]
+            filename = filename.split("</a>")[0]
+            files.append(filename)
+    del content
+    dates = [dt.strptime(i.split("_")[0],'%Y%m%d%H%M') for i in files]
+    diff = [(time-i).total_seconds()/3600 for i in dates]
+    diff = [i for i in diff if i >= 0]
+
+    #Continue if less than 24 hours difference
+    if np.nanmin(diff) <= 30:
+        two_date = dates[diff.index(np.nanmin(diff))].strftime('%Y%m%d%H%M')
+
+        #Retrieve NHC shapefiles for development areas
+        shapefiles = {}
+        for name in ['areas','lines','points']:
+
+            try:
+                #Read in shapefile zip from NHC
+                url = f'https://www.nhc.noaa.gov/gis/gtwo/archive/{two_date}_gtwo.zip'
+                request = urllib.request.Request(url)
+                response = urllib.request.urlopen(request)
+                file_like_object = BytesIO(response.read())
+                tar = zipfile.ZipFile(file_like_object)
+
+                #Get file list (points, areas)
+                members = '\n'.join([i for i in tar.namelist()])
+                nums = "[0123456789]"
+                search_pattern = f'gtwo_{name}_20{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}{nums}.shp'
+                pattern = re.compile(search_pattern)
+                filelist = pattern.findall(members)
+                files = []
+                for file in filelist:
+                    if file not in files: files.append(file.split(".shp")[0]) #remove duplicates
+
+                #Retrieve necessary components for shapefile
+                members = tar.namelist()
+                members_names = [i for i in members]
+                data = {'shp':0,'dbf':0,'prj':0,'shx':0}
+                for key in data.keys():
+                    idx = members_names.index(files[0]+"."+key)
+                    data[key] = BytesIO(tar.read(members[idx]))
+
+                #Read in shapefile
+                orig_reader = shapefile.Reader(shp=data['shp'], dbf=data['dbf'], prj=data['prj'], shx=data['shx'])
+                shapefiles[name] = BasicReader(orig_reader)
+            except:
+                shapefiles[name] = None
+    else:
+        shapefiles = {'areas':None,'lines':None,'points':None}
+    
+    return shapefiles
+
 def nhc_cone_radii(year,basin,forecast_hour=None):
     
     r"""
@@ -435,7 +573,7 @@ def nhc_cone_radii(year,basin,forecast_hour=None):
     year : int
         Valid year for cone of uncertainty radii.
     basin : str
-        Basin for cone of uncertainty radii. If basin is invalid, return value will be an empty dict.
+        Basin for cone of uncertainty radii. If basin is invalid, return value will be an empty dict. Please refer to :ref:`options-domain` for available basin options.
     forecast_hour : int or list, optional
         Forecast hour(s) to retrieve the cone of uncertainty for. If empty, all available forecast hours will be retrieved.
     
@@ -508,7 +646,7 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
     forecast : dict
         Dictionary containing forecast data
     basin : str
-        Basin for cone of uncertainty radii.
+        Basin for cone of uncertainty radii. Please refer to :ref:`options-domain` for available basin options.
     shift_lons : bool, optional
         If true, grid will be shifted to +0 to +360 degrees longitude. Default is False (-180 to +180 degrees).
     cone_days : int, optional
@@ -522,6 +660,10 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
     -------
     dict or xarray.Dataset
         Depending on `return_xarray`, returns either a dictionary or an xarray Dataset containing the gridded cone of uncertainty and its accompanying attributes.
+    
+    Notes
+    -----
+    Forecast dicts can be retrieved for realtime storm objects using ``RealtimeStorm.get_forecast_realtime()``, and for archived storms using ``Storm.get_nhc_forecast_dict()``.
     """
     
     #Check forecast dict has all required keys
@@ -529,9 +671,9 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
     if False in check_dict:
         raise ValueError("forecast dict must contain keys 'fhr', 'lat', 'lon' and 'init'. You may retrieve a forecast dict for a Storm object through 'storm.get_operational_forecasts()'.")
     
-    #Determine storm basin
-    if 'basin' not in forecast.keys():
-        forecast['basin'] = basin
+    #Check forecast basin
+    if basin not in constants.ALL_BASINS:
+        raise ValueError("basin cannot be identified.")
 
     #Retrieve cone of uncertainty year
     if cone_year is None:
@@ -544,8 +686,8 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
         warnings.warn(f"No cone information is available for the requested year. Defaulting to 2008 cone.")
 
     #Retrieve cone size and hours for given year
-    if forecast['basin'] in ['north_atlantic','east_pacific']:
-        output = nhc_cone_radii(cone_year,forecast['basin'])
+    if basin in ['north_atlantic','east_pacific']:
+        output = nhc_cone_radii(cone_year,basin)
         cone_climo_hr = [k for k in output.keys()]
         cone_size = [output[k] for k in output.keys()]
     else:
@@ -699,7 +841,11 @@ def generate_nhc_cone(forecast,basin,shift_lons=False,cone_days=5,cone_year=None
     #Return if no cone specified
     if cone_size == 0:
         return_dict = {'center_lon':interp_lon,'center_lat':interp_lat}
-        return return_dict
+        if return_xarray:
+            import xarray as xr
+            return xr.Dataset(return_dict)
+        else:
+            return return_dict
 
     #Interpolate cone radius temporally
     cone_climo_hr = cone_climo_hr[:cone_day_cap]
@@ -809,6 +955,98 @@ def category_label_to_wind(category):
     else:
         return category_to_wind(5)
 
+def dynamic_map_extent(min_lon,max_lon,min_lat,max_lat):
+
+    r"""
+    Sets up a dynamic map extent with an aspect ratio of 3:2 given latitude and longitude bounds.
+
+    Parameters:
+    -----------
+    min_lon : float
+        Minimum longitude bound.
+    max_lon : float
+        Maximum longitude bound.
+    min_lat : float
+        Minimum latitude bound.
+    max_lat : float
+        Maximum latitude bound.
+
+    Returns:
+    --------
+    list
+        List containing new west, east, north, south map bounds, respectively.
+    """
+
+    #Get lat/lon bounds
+    bound_w = min_lon+0.0
+    bound_e = max_lon+0.0
+    bound_s = min_lat+0.0
+    bound_n = max_lat+0.0
+
+    #If only one coordinate point, artificially induce a spread
+    if bound_w == bound_e:
+        bound_w = bound_w - 0.6
+        bound_e = bound_e + 0.6
+    if bound_s == bound_n:
+        bound_n = bound_n + 0.6
+        bound_s = bound_s - 0.6
+
+    #Function for fixing map ratio
+    def fix_map_ratio(bound_w,bound_e,bound_n,bound_s,nthres=1.45):
+        xrng = abs(bound_w-bound_e)
+        yrng = abs(bound_n-bound_s)
+        diff = float(xrng) / float(yrng)
+        if diff < nthres: #plot too tall, need to make it wider
+            goal_diff = nthres * (yrng)
+            factor = abs(xrng - goal_diff) / 2.0
+            bound_w = bound_w - factor
+            bound_e = bound_e + factor
+        elif diff > nthres: #plot too wide, need to make it taller
+            goal_diff = xrng / nthres
+            factor = abs(yrng - goal_diff) / 2.0
+            bound_s = bound_s - factor
+            bound_n = bound_n + factor
+        return bound_w,bound_e,bound_n,bound_s
+
+    #First round of fixing ratio
+    bound_w,bound_e,bound_n,bound_s = fix_map_ratio(bound_w,bound_e,bound_n,bound_s,1.45)
+
+    #Adjust map width depending on extent of storm
+    xrng = abs(bound_e-bound_w)
+    yrng = abs(bound_n-bound_s)
+    factor = 0.1
+    if min(xrng,yrng) < 15.0:
+        factor = 0.2
+    if min(xrng,yrng) < 12.0:
+        factor = 0.4
+    if min(xrng,yrng) < 10.0:
+        factor = 0.6
+    if min(xrng,yrng) < 8.0:
+        factor = 0.75
+    if min(xrng,yrng) < 6.0:
+        factor = 0.9
+    bound_w = bound_w-(xrng*factor)
+    bound_e = bound_e+(xrng*factor)
+    bound_s = bound_s-(yrng*factor)
+    bound_n = bound_n+(yrng*factor)
+
+    #Second round of fixing ratio
+    bound_w,bound_e,bound_n,bound_s = fix_map_ratio(bound_w,bound_e,bound_n,bound_s,1.45)
+
+    #Return map bounds
+    return bound_w,bound_e,bound_s,bound_n
+
+def read_url(url,split=True,subsplit=True):
+    
+    f = urllib.request.urlopen(url)
+    content = f.read()
+    content = content.decode("utf-8")
+    if split: content = content.split("\n")
+    if subsplit: content = [(i.replace(" ","")).split(",") for i in content]
+    f.close()
+    
+    return content
+
 class Distance:
     
     def __init__(self,dist,units='kilometers'):
@@ -888,3 +1126,118 @@ class great_circle(Distance):
 
         #Return great circle distance
         return self.RADIUS * d
+
+r"""
+This is a modified version of Cartopy's shapereader functionality, specified to directly read in an already-existing
+Shapely object as opposed to expecting a local shapefile to be read in.
+"""
+import shapely.geometry as sgeom
+import shapefile
+
+class Record:
+    """
+    A single logical entry from a shapefile, combining the attributes with
+    their associated geometry.
+
+    """
+    def __init__(self, shape, attributes, fields):
+        self._shape = shape
+
+        self._bounds = None
+        # if the record defines a bbox, then use that for the shape's bounds,
+        # rather than using the full geometry in the bounds property
+        if hasattr(shape, 'bbox'):
+            self._bounds = tuple(shape.bbox)
+
+        self._geometry = None
+        """The cached geometry instance for this Record."""
+
+        self.attributes = attributes
+        """A dictionary mapping attribute names to attribute values."""
+
+        self._fields = fields
+
+    def __repr__(self):
+        return f'<Record: {self.geometry!r}, {self.attributes!r}, <fields>>'
+
+    def __str__(self):
+        return f'Record({self.geometry}, {self.attributes}, <fields>)'
+
+    @property
+    def bounds(self):
+        """
+        The bounds of this Record's :meth:`~Record.geometry`.
+
+        """
+        if self._bounds is None:
+            self._bounds = self.geometry.bounds
+        return self._bounds
+
+    @property
+    def geometry(self):
+        """
+        A shapely.geometry instance for this Record.
+
+        The geometry may be ``None`` if a null shape is defined in the
+        shapefile.
+
+        """
+        if not self._geometry and self._shape.shapeType != shapefile.NULL:
+            self._geometry = sgeom.shape(self._shape)
+        return self._geometry
+
+class BasicReader:
+    """
+    Provide an interface for accessing the contents of a shapefile.
+
+    The primary methods used on a Reader instance are
+    :meth:`~Reader.records` and :meth:`~Reader.geometries`.
+
+    """
+    def __init__(self, reader):
+        # Validate the filename/shapefile
+        self._reader = reader
+        if reader.shp is None or reader.shx is None or reader.dbf is None:
+            raise ValueError("Incomplete shapefile definition "
+                             "in '%s'." % filename)
+
+        self._fields = self._reader.fields
+
+
+    def close(self):
+        return self._reader.close()
+
+    def __len__(self):
+        return len(self._reader)
+
+    def geometries(self):
+        """
+        Return an iterator of shapely geometries from the shapefile.
+
+        This interface is useful for accessing the geometries of the
+        shapefile where knowledge of the associated metadata is not necessary.
+        In the case where further metadata is needed use the
+        :meth:`~Reader.records`
+        interface instead, extracting the geometry from the record with the
+        :meth:`~Record.geometry` method.
+
+        """
+        to_return = []
+        for shape in self._reader.iterShapes():
+            # Skip the shape that can not be represented as geometry.
+            if shape.shapeType != shapefile.NULL:
+                to_return.append(sgeom.shape(shape))
+        return to_return
+
+    def records(self):
+        """
+        Return an iterator of :class:`~Record` instances.
+
+        """
+        # Ignore the "DeletionFlag" field which always comes first
+        to_return = []
+        fields = self._reader.fields[1:]
+        for shape_record in self._reader.iterShapeRecords():
+            attributes = shape_record.record.as_dict()
+            to_return.append(Record(shape_record.shape, attributes, fields))
+        return to_return
